/* This program converts SAC data files to AH
 *
 *			Latest Revision; Nov. 24, 1987
 *				By; Tom Boyd
 *
 *			Updated to read and write xdr ah files; Sep. 27, 1989
 *				By; Ken Howard
 *
 */
#include <stdio.h>
#include <rpc/rpc.h>
#include <math.h>
#include "../ahhead.h"

#define SAC_HEAD_SIZE	632		/* size of SAC header in bytes		*/
#define UNDEFINED_FLOAT -12345.0	/* value of undefined float in SAC file	*/
#define UNDEFINED_INT -12345		/* value of undefined int in SAC file	*/

char *progname;
main (argc,argv)
	int argc;
	char *argv[];
{
	ahhed ahhead;
	XDR	xdr_out;
	long count;
	char *calloc(),sachead[SAC_HEAD_SIZE];
	float *data, temp;
	void fill_ah_head();

	progname=argv[0];
	xdrstdio_create(&xdr_out, stdout, XDR_ENCODE);

	if((out_is_tty() != 0)||(in_is_tty() != 0))
	{
		fprintf(stderr,"Usage: %s < stdin > stdout\n",progname);
		exit(-1);
	}

	while(fread(sachead,1,SAC_HEAD_SIZE,stdin) == SAC_HEAD_SIZE)
	{

		/* get null ah header */

		get_null_head(&ahhead);

		/* fill ah header */

		fill_ah_head(&ahhead,sachead);

		/* allocate data space */

		if((data=(float *)calloc((unsigned)(ahhead.record.ndata),size(&ahhead))) == NULL)
		{
			fprintf(stderr,"Error allocating space in %s\n",progname);
			exit(-2);
		}

		/* read data */

		switch ( ahhead.record.type )
		{
			case FLOAT:		/* Time Series Data---Stored as FLOAT in ah */
				if(fread(data,4,ahhead.record.ndata,stdin) != ahhead.record.ndata)
				{
					fprintf(stderr,"Error in %s reading data\n",progname);
					exit(-1);
				}
				break;
			case COMPLEX:		/* Spectra---Stored as COMPLEX in ah */
				for(count=0;count<ahhead.record.ndata;count++)
				{
					if(fread((data+count*2),4,1,stdin) != ahhead.record.ndata)
					{
						fprintf(stderr,"Error in %s reading data\n",progname);
						exit(-1);
					}
				}
				for(count=0;count<ahhead.record.ndata;count++)
				{
					if(fread((data+count*2+1),4,1,stdin) != ahhead.record.ndata)
					{
						fprintf(stderr,"Error in %s reading data\n",progname);
						exit(-1);
					}
				}
				break;
			default:
				fprintf(stderr,"Error in %s: %d is an unknown data type\n",progname, 
					ahhead.record.type);
				break;
		}

		/* modify log entry */

		logger("sac;",&ahhead);

		/* write ah header and data */

		if(xdr_puthead(&ahhead, &xdr_out) != 1) 
		{
			fprintf(stderr,"Error writing header in %s\n",progname);
			exit(-3);
		}
		if(xdr_putdata(&ahhead,(char *)data, &xdr_out) < 0)
		{
			fprintf(stderr,"Error writing data in %s\n",progname);
			exit(-3);
		}

		/* free array space */

		cfree((char *)data);

	}
	xdr_destroy(&xdr_out);
}

void fill_ah_head(ahhead,sachead)
	ahhed *ahhead;
	char *sachead;
{
	float *f_pt;
	long *i_pt, jday;
	char *c_pt;
	float e_time;
	double htoe(), epoch;


	/* fill ahheader */

	f_pt=(float *)sachead;

	ahhead->record.delta= *f_pt;
	ahhead->record.maxamp= ( fabs(*(f_pt+1)) > fabs(*(f_pt+2)) ) ? fabs(*(f_pt+1)) : fabs(*(f_pt+2));

	/* get record offset time */

	e_time=0.0;
	if( *(f_pt+7) != UNDEFINED_FLOAT)
		e_time= *(f_pt+7);
	
	/* get event and station location information */

	if( *(f_pt+31) != UNDEFINED_FLOAT)
		ahhead->station.slat= *(f_pt+31);
	if( *(f_pt+32) != UNDEFINED_FLOAT)
		ahhead->station.slon= *(f_pt+32);
	if( *(f_pt+33) != UNDEFINED_FLOAT)
		ahhead->station.elev= *(f_pt+33);
	if( *(f_pt+35) != UNDEFINED_FLOAT)
		ahhead->event.lat= *(f_pt+35);
	if( *(f_pt+36) != UNDEFINED_FLOAT)
		ahhead->event.lon= *(f_pt+36);
	if( *(f_pt+38) != UNDEFINED_FLOAT)
		ahhead->event.dep= *(f_pt+38);

	i_pt=(int *)(f_pt+70);
	
	/* get reference time */

	if( *(i_pt) != UNDEFINED_INT)
		ahhead->record.abstime.yr= *i_pt;
	if( *(i_pt+1) != UNDEFINED_INT)
	{
		jday= *(i_pt+1);	/* given Julian day get Month and Day */
		month_day(&(ahhead->record.abstime),jday);
	}
	if( *(i_pt+2) != UNDEFINED_INT)
		ahhead->record.abstime.hr= (short)*(i_pt+2);
	if( *(i_pt+3) != UNDEFINED_INT)
		ahhead->record.abstime.mn= (short)*(i_pt+3);
	if( *(i_pt+4) != UNDEFINED_INT && *(i_pt+5) != UNDEFINED_INT )
		ahhead->record.abstime.sec= (float)*(i_pt+4) + (float)*(i_pt+5)/1000.0;

	/* if there is a record offset time add it to the reference time */

	if(e_time != 0.0)
	{
		epoch=htoe(&(ahhead->record.abstime));
		epoch+=(double)e_time;
		etoh(&(ahhead->record.abstime),epoch);
	}

	/* get number of data points */

	ahhead->record.ndata= *(i_pt+9);
	
	/* get data type */

	switch ( *(i_pt+15) )
	{
		case 1:		/* Time Series Data---Stored as FLOAT in ah */
			ahhead->record.type=FLOAT;
			break;
		case 2:		/* Spectra---Stored as COMPLEX in ah */
			ahhead->record.type=COMPLEX;
			strcpy(&(ahhead->event.ecomment[0]),"Time_Series_Spectra--Real_and_Imaginary");
			break;
		case 3:		/* Spectra---Stored as COMPLEX in ah */
			ahhead->record.type=COMPLEX;
			strcpy(&(ahhead->event.ecomment[0]),"Time_Series_Spectrum---Amplitude_Phase");
			break;
		case 4:		/* general x y data----Not supported in ah */
			fprintf(stderr,"Error in %s: x y data not currently supported\n",progname);
			exit(-1);
			break;
		default:
			fprintf(stderr,"Error in %s: %d is an unknown data type\n",progname, *(i_pt+15));
			break;
	}

	/* set comment to show data type */

	switch ( *(i_pt+16) )
	{
		case 1:		/* Unknown */
			break;
		case 2:		/* Displacement in nm */
			strcpy(&(ahhead->record.rcomment[0]),"Displacement in nm");
			break;
		case 3:		/* Velocity in nm/sec */
			strcpy(&(ahhead->record.rcomment[0]),"Velocity in nm/sec");
			break;
		case 4:		/* Velocity in volts */
			strcpy(&(ahhead->record.rcomment[0]),"Velocity in Volts");
			break;
		case 5:		/* Acceleration in nm/sec/sec */
			strcpy(&(ahhead->record.rcomment[0]),"Acceleration in nm/sec/sec");
			break;
		default:
			break;
	}

	c_pt=(char *)(i_pt+40);

	/* station name */

	*(c_pt+5)='\0';
	strcpy(&(ahhead->station.code[0]), c_pt);

	/* component name */

	*(c_pt+165)='\0';
	strcpy(&(ahhead->station.chan[0]), (c_pt+160));

	/* instrument type */

	*(c_pt+189)='\0';
	strcpy(&(ahhead->station.stype[0]), (c_pt+184));
}

